from .calc import confidence, support
import pandas as pd
import itertools

def has_duplicates(iterable):
  l = list(itertools.chain(*iterable)) # in case iterable is an iterator
  return len(set(l)) != len(l)

def permutations(iterable, r=None):
  pool = tuple(iterable)
  n = len(pool)
  r = n if r is None else r
  if r > n:
    return
  indices = list(range(n))
  cycles = list(range(n, n-r, -1))
  permu = [pool[i] for i in indices[:r]]
  if not(has_duplicates(permu)):
    yield permu

  while n:
    for i in reversed(range(r)):
      cycles[i] -= 1
      if cycles[i] == 0:
        indices[i:] = indices[i+1:] + indices[i:i+1]
        cycles[i] = n - i
      else:
        j = cycles[i]
        indices[i], indices[-j] = indices[-j], indices[i]
        permu = [pool[i] for i in indices[:r]]
        if not(has_duplicates(permu)):
          yield permu
        break
    else:
      return

def getPermutations(iterable, r=None):
    return list(permutations(iterable,r))

def association_rule(itemset = [], min_confidence = 0.5):
    asso = getPermutations(itemset,2)
    table = []
    for item in asso:
        conf = confidence(item[0], item[1])
        if conf >= min_confidence:
            table.append([str(item[0])+" --> "+str(item[1]),item[0], item[1], support(item[0]), support(item[1]), conf])
    return pd.DataFrame(table, columns=['Notasi','Antecedent', 'Consequents', 'Antecedent Support', 'Consequents Support', 'Confidence']) 



from .calc import support, itemFreq
import itertools
import numpy as np
import pandas as pd

dataset = []

def getSupportwithFilter(dataset, min_sup = 0.5):
  result = []
  for data in dataset:
    supp = support(data) if isinstance(data, list) else support([data])
    if supp >= min_sup:
      result.append([data, itemFreq(data), supp])
  return result

def getCombination(arr, k):
  return list(map(list, itertools.combinations(arr, k)))

def getUnique(arr):
  return list(set(x for l in arr for x in l))

def apriori(data, min_support=0.5):
  global dataset
  dataset = getUnique(data)
  all_support = []

  data_filtered = np.array(getSupportwithFilter(dataset, min_support))[:,0]

  for i in range(1,4):
    #Get Combination
    combi = getCombination(data_filtered, i)
    all_support.extend(getSupportwithFilter(combi, min_support))
  return pd.DataFrame(all_support, columns=['ItemSet', 'Count', 'Support'])


# M Tafaquh Fiddin Al Islami 2020
# TieF AI Tafaquh Fiddin Al Islami Library
# Author: M Tafaquh Fiddin Al Islami
#
# License: MIT

from .association.apriori import apriori
from .association.association_rule import association_rules

__all__ = ["apriori", "association_rules"]


# M Tafaquh Fiddin Al Islami 2020
# TieF AI Tafaquh Fiddin Al Islami Library
# Author: M Tafaquh Fiddin Al Islami
#
# License: MIT


from distutils.core import setup
import setuptools

setup(
  name = 'tief',         # How you named your package folder (MyLib)
  packages = setuptools.find_packages(),   # Chose the same as "name"
  version = '0.1.1',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'TIEF.ai is an AI library for research and study purpose. This library is very easy to use. If you found some lack of performance or some issues, just tell me. I ll upgrade it as soon as possible.',   # Give a short description about your library
  author = 'M Tafaquh Fiddin Al Islami',                   # Type in your name
  author_email = 'tafaquh.fiddinal@gmail.com',      # Type in your E-Mail
  url = 'https://github.com/user/reponame',   # Provide either the link to your github or to your website
  download_url = 'https://github.com/tafaquh/tief/archive/0.1.tar.gz',    # I explain this later on
  keywords = ['AI', 'Machine Learning', 'TIEF', 'Library'],   # Keywords that define your package best
  install_requires=[            # I get to this in a second
    'numpy',
    'pandas',
  ],
  classifiers=[
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
  ],
  python_requires='>=3.6',
)



from .apriori import dataset

def itemFreq(_item):
  """
    Return support value
    _item: list of string, ex ['123'] or ['123', '124']
    dataset: list of list of string, ex [['123'], ['123', '124']]
  """
  return len([row for row in dataset if set(_item).issubset(row)])

def support(_item):
  """
    Return support value
    _item: list of string, ex ['123'] or ['123', '124']
    dataset: list of list of string, ex [['123'], ['123', '124']]
  """
  return itemFreq(_item)/len(dataset)

def confidence(_item, next_item):
  """
    Return confidence value
    _item: list of string, ex ['123'] or ['123', '124']
    next_item: list of string, ex ['123'] or ['123', '124']
  """
  join = _item + next_item
  return support(join)/support(_item)
